/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.functions;

import java.util.ArrayList;
import java.util.List;


import com.carlos.math.numerico.fourier.Coeficent;
import com.carlos.math.numerico.fourier.Controlador;
import com.carlos.math.numerico.fourier.FourierSerie;
import com.carlos.math.numerico.rkuta.New65f;

import com.carlos.math.numerico.util.VariabledFParser;
import com.numerico.algebralineal.Util;

public class VariabledFunction implements Cloneable{
	
	protected VariabledFParser parser;
	protected String[] funcion;
	protected int dimension;
	
	public VariabledFunction(String funcion)
	{
		
		this(new String[]{funcion});
	
	}
	
	public VariabledFunction()
	{
	}
	private void addDefaultVariables(int index)
	{
		this.addVariable("x", index);
		this.addVariable("y", index);
		this.addVariable("z", index);
		this.addVariable("w", index);
		this.addVariable("u", index);
		this.addVariable("v", index);
		this.addVariable("s", index);
		this.addVariable("t", index);
	}
	
	public void addVariableForAll(String varName)
	{
		for(int k=0; k< this.funcion.length; k++)
		{
			addVariable(varName, k);
		}
	}
	
	public VariabledFunction(List<String> funcion) 
	{
		this.dimension = funcion.size();
		parser = new VariabledFParser(dimension);
		
		int j =0;
		String [] s = new String[funcion.size()];
		for(String f: funcion)
		{
			f = arrumateFunction(f);
			parser.addFunction(f, j);
			s[j] = f;
			addDefaultVariables(j);
			j++;
		}
			
		this.funcion = s;
		
	}
	
	public VariabledFunction(String[] funcion) 
	{
		this.dimension = funcion.length;
		parser = new VariabledFParser(dimension);
		
		int j =0;
		String s = null;
		for(String f: funcion)
		{
			f = arrumateFunction(f);
			parser.addFunction(f, j);
			addDefaultVariables(j);
			j++;
		}
		
		
		this.funcion = funcion;
		
	}
	
	private String arrumateFunction(String s)
	{
		s = s.toLowerCase();
		s = s.replaceAll(",", ".");
		/*
		s = s.replaceAll("[", "(");
		s = s.replaceAll("]", ")");
		s = s.replaceAll("{", "(");
		s = s.replaceAll("}", ")");
		
		s = s.replaceAll("x(", "x*(");
		s = s.replaceAll("y(", "y*(");
		s = s.replaceAll(")y", ")*y");
		s = s.replaceAll(")x", ")*x");
		s = s.replaceAll("t(", "t*(");
		s = s.replaceAll("t(", ")*t");
		s = s.replaceAll("z(", "z*(");
		s = s.replaceAll("z(", ")*z");
		*/
		s = s.replaceAll("\\u221e", "200000");
		s = s.replaceAll("xy", "(x*y)");
		s = s.replaceAll("x y", "(x*y)");
		s = s.replaceAll("y x", "(y*x)");
		s = s.replaceAll("yx", "(y*x)");
		
		s = s.replaceAll("sinx", "(sin(x))");
		s = s.replaceAll("sin x", "(sin(x))");
		s = s.replaceAll("cosx", "(cos(x))");
		s = s.replaceAll("cos x", "(cos(x))");
		s = s.replaceAll("tanx", "(tan(x))");
		s = s.replaceAll("tan x", "(tan(x))");
		
		s = s.replaceAll("logx", "(log(x))");
		s = s.replaceAll("log x", "(log(x))");
		
		return s;
	}
	
	
	
	public void addVariable(String name, int index)
	{
		parser.addVariable(name, index);
	}
	

	public VariabledFunction getFunctionWithVariableSetted(String variable, double value)
	{
		String [] s = new String[this.dimension];
		for(int k=1; k<= this.getDimension(); k++)
		{
			s[k-1] = this.getFunctionAt(k-1).replaceAll(variable, Double.toString(value));
		}
		return new VariabledFunction(s);
	}
	

	public void addFunctionStr(String function)
	{
		VariabledFParser parser2 = (VariabledFParser)this.parser.clone();
		
		this.dimension++;
		parser = new VariabledFParser (dimension);
		for(int k=0; k< this.dimension-1; k++)
		{
			parser.addConfiguration(parser2.getConfig()[k], k);
		}
		parser.addFunction(function);
	}
	
	public VariabledFunction convolues(String variable, String varRest, VariabledFunction otherFunction)
	{
		String [] s = new String[dimension];
		
		for(int k =0; k< this.dimension; k++)
		{
			s[k] = "(" + otherFunction.getFunctionAt(k) + ")*(" + this.getFunctionAt(k).replaceAll(variable, "(" + variable + " - " + varRest + ")") + ")";
		}
		VariabledFunction convolution = new VariabledFunction(s);
		return convolution;
	}
	public static void main(String[] args) throws Exception
	{
		String s = "-inf";
		s = s.replace("inf", "200000");
		System.out.println(s);
		VariabledFunction f = new VariabledFunction("x^2");
		double d = f.integrateSemiInfinite(1, false);
		System.out.println(d);
		/*
		VariabledFunction f = new VariabledFunction("x^2");
		System.out.println(f.integrateSemiInfinite(-1, true));
		
		System.exit(0);
		 f = new VariabledFunction("(x)*(cos(y))");
		f.addVariable("x",0);
		f.addVariable("y",0);
		System.out.println("f em x=1, y=2 " + f.f(new double[]{10.0, 10})[0]);
		System.out.println("f[0] " + f.getFunctionAt(0));
		System.exit(0);
		try
		{
			//System.out.println(f.calculatesAninflectionsPont(0.1, new double[]{1, 1}, 0.00005, true)[0]);
			System.out.println("Divergence at 10 " + f.divergence(new double []{10, 10}));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		*/
	}
	
	public boolean isNullFunction()
	{
		boolean retured = false;
		for(int k=0; k< this.parser.getDimension(); k++)
		{
			if(parser == null || this.parser.getFunctionAt(k) == null || this.parser.getFunctionAt(k).equals("null"))
			{
				return true;
			}
		}
		return retured;
	}
	public String getFunctionAt(int index)
	{
		return parser != null && this.parser.getFunctionAt(index) != null && !this.parser.getFunctionAt(index).equals("null")  ? this.parser.getFunctionAt(index) : null;
	}
	
	public double [] calculatesAninflectionsPont(double step, double[] xInicial, double precision, boolean minimize) throws Exception
	{
		double x[] = new double[xInicial.length];
		for(int k=0; k< xInicial.length; k++)
		{
			x[k] = xInicial[k];
		}
		double v[] = new double[x.length];
		double [] y = new double[this.dimension];
		int add = minimize ? -1 : 1;
		double norma=0;
		double h0 = step;
		int j = 0;
		int i=0;
		do
		{
			v = this.gradient(x);
			
			x = Util.vectorialSum(x, Util.scalarMultiply(Util.scalarMultiply(v, add), h0));
			y = this.f(x);
			
			j++;
			
			if(Util.normaLInfinite(v)>norma)
			{
				System.in.read();
			}
			norma = Util.normaLInfinite(v);
			i++;
		}
		while(Util.normaLInfinite(v) > precision && i< Util.ITERATIONS_LIMIT);
		
		//while(Util.normaLp(v, 2) > precision);
		
		return y;
	}
	
	/*Solo aplicable cuando la dimesion es 1!!!*/
	public double[] gradient(double[] y) throws Exception
	{
		double [] dReturn = new double[y.length]; 
		
		for(int j= 0; j< dReturn.length; j++)
		{
			
			dReturn[j] =  this.derivatesPartial(y, j)[0];
		}
		return dReturn;
	}
	
	public VariabledFunction multiplies(VariabledFunction f)
	{
		String [] s = new String[this.getDimension()];
		if(f.getDimension() != this.getDimension())
		{
			return null;
		}
		for(int k=0; k< this.getDimension(); k++)
		{
			s[k] = "(" + this.getFunctionAt(k) + ") * (" + f.getFunctionAt(k) + ")";
		}
		return new VariabledFunction(s);
		
	}
	
	public VariabledFunction differenciate(String variable)
	{
		return this.parser.differenciate(variable);
	}
	
	public VariabledFunction differenciateToX()
	{
		VariabledFunction f = this.differenciate("x");
		if(f.isCero())
		{
			f = this.differenciate("t");
			
		}
		return f;
	}
	
	public boolean isCero()
	{
		return this.parser.isCero();
	}
	
	public VariabledFunction multiply(VariabledFunction f)
	{
		String [] s = new String[dimension]; 
		for(int k=0; k< this.dimension; k++)
		{	
			if(k<f.dimension)
			{
				s[k] = "(" + this.getFunctionAt(k) + ")*(" + f.getFunctionAt(k) + ")";
			}
			else
			{
				s[k] = this.getFunctionAt(k);
			}
		}
		return new VariabledFunction(s);
	}
	
	public double[] derivatesPartial(double y[], int direction) throws Exception
	{
		double[] d = Util.scalarMultiply(Util.ei(y.length, direction), Util.CONSTANTE_3);
		d = Util.vectorialRest(this.f(Util.vectorialSum(d, y)), this.f(Util.vectorialRest(y, d)));
		d = Util.scalarMultiply(d, 1/(2 * Util.CONSTANTE_3));
		return d;
	}
	
	/*Solo aplicable cuando la dimension es 1!!!*/
	public double derivatesSecond(double y[]) throws Exception
	{
		return this.derivatesPartialSecond(y, 0);
	}
	public double derivates(double x) throws Exception
	{
		return (this.f(x - 2 * Util.CONSTANTE_3) - 8 * this.f(x - Util.CONSTANTE_3) + 8 * this.f(x + Util.CONSTANTE_3) - this.f(x + 2 * Util.CONSTANTE_3) ) / ( 12*Util.CONSTANTE_3 );
	}
	/*Solo aplicable cuando la dimension es 1!!!*/
	public double divergence(double[] y) throws Exception
	{
		double dReturn = 0;
		for(int j= 0; j< y.length; j++)
		{
			dReturn += this.derivatesPartial(y, j)[0];
		}
		return dReturn;
	}
	
	public double f(int x) throws Exception
	{
		return this.f(new Double(x));
	}
	
	public double f(double x) throws Exception
	{
		return this.getParserForX(x).evaluate(joinFirstCoortinateToVector(x, new double[1]))[0];
	}
	
	public double [] f(double[] y) throws Exception
	{
		return this.getParser().evaluate(y);
	}
	
	public double [] f(double x, double[] y) 
	{
		//System.out.println(SimplexUtil.printVector2("Evaluando en x="+x +"  ", y));
		//System.out.println(SimplexUtil.printVector2("Evaluando en ", joinFirstCoortinateToVector(x, y)));
		return this.getParser().evaluate(joinFirstCoortinateToVector(x, y));
	}
	
	private double[] joinFirstCoortinateToVector(double x, double[] y)
	{
		double [] d = new double[y.length+1];
		d[0] = x;
		if(d.length == 1)
			return d;
		for(int k = 0; k< y.length; k++)
		{
			d[k+1] = y[k];
		}
		return d;
	}
	
	/*get Normal Vector to a Surface, if normed the norm vector is 1*/
	public VariabledFunction getNormalVector(int orientation, boolean normed) throws Exception
	{
		VariabledFunction partialX = this.differenciate("x");
		VariabledFunction partialY = this.differenciate("y");
		VariabledFunction vectorial;
		if(orientation ==1)
		{
			vectorial= partialX.vectorialProduct(partialY);
		}
		else
		{
			vectorial= partialY.vectorialProduct(partialX);
		}
		
		if(!normed) return vectorial;
		
		VariabledFunction scalar = vectorial.getAsL2Operator();
		
		String [] s = new String[this.getDimension()];
		for(int k =0; k< vectorial.getDimension(); k++)
		{
			s[k] = "(" + vectorial.getFunctionAt(k) + ")/(" + scalar.getFunctionAt(0)+")";
		}
		
		return new VariabledFunction(s);
	}
	
	public VariabledFunction vectorialProduct(VariabledFunction f) throws Exception
	{
		String [] s = new String[this.getDimension()];
		
		switch (this.getDimension()) {
		case 2:
			s[0] = "(" + f.getFunctionAt(1) + ")*(" + this.getFunctionAt(0) + ")";
			s[1] = "-1*((" + f.getFunctionAt(0) + ")*(" + this.getFunctionAt(1) + "))";
			
		case 3:
			s[0] = "(" + f.getFunctionAt(2) + ")*(" + this.getFunctionAt(1) + ")";
			s[0] += "-1*((" + f.getFunctionAt(1) + ")*(" + this.getFunctionAt(2) + "))";
			
			s[1] = "(" + f.getFunctionAt(0) + ")*(" + this.getFunctionAt(2) + ")";
			s[1] += "-1*((" + f.getFunctionAt(2) + ")*(" + this.getFunctionAt(0) + "))";
			
			s[2] = "(" + f.getFunctionAt(1) + ")*(" + this.getFunctionAt(0) + ")";
			s[2] += "-1*((" + f.getFunctionAt(0) + ")*(" + this.getFunctionAt(1) + "))";
			
		default:
			s = new String[1];
			s[0] = "(" + f.getFunctionAt(0) + ")*(" + this.getFunctionAt(0) + ")";
			
		}
		
		return new VariabledFunction(s);
	}
	/*Solo aplicable cuando la dimension es 1!!!*/
	public double derivatesPartialSecond(double y[], int direction) throws Exception
	{
		double [] d = Util.scalarMultiply(Util.ei(y.length, direction), -2 * Util.CONSTANTE_3);
		double [] d2 = Util.scalarMultiply(Util.ei(y.length, direction), - Util.CONSTANTE_3);
		double [] d3 = Util.scalarMultiply(Util.ei(y.length, direction), Util.CONSTANTE_3);
		double [] d4 = Util.scalarMultiply(Util.ei(y.length, direction), 2 * Util.CONSTANTE_3);
		return (this.derivatesPartial(Util.vectorialSum(y, d), direction)[0] - 8 * this.derivatesPartial(Util.vectorialSum(y, d2), direction)[0] + 8 * this.derivatesPartial(Util.vectorialSum(y, d3), direction)[0] - this.derivatesPartial(Util.vectorialSum(y, d4), direction)[0]) / ( 12*Util.CONSTANTE_3 );
	}
	
	/*
	 * Calculates the point where an inflection point holds beggining by a x0 initial.
	 * precision is the accuracy
	 * step is the size steps for the iterations
	*/
	
	public InflectedPoint calculatesWhereAninflectionsPontHolds(double step, double[] xInicial, double precision, boolean minimize) throws Exception
	{
		double x[] = xInicial;
		double v[] = new double[x.length];
		double [] y = new double[this.dimension];
		double h0 = step;
		int add = minimize ? -1 : 1;
		int j = 0;
		int ii = 0;
		do
		{
			v = this.gradient(x);
			x = Util.vectorialSum(x, Util.scalarMultiply(Util.scalarMultiply(v, add), h0));
			y = this.f(x);
			j++;
			ii++;
		}
		while(Util.normaLInfinite(v) > precision  && ii< Util.ITERATIONS_LIMIT);
		InflectedPoint i = new InflectedPoint(x, this);
		
		return i;
	}
	/*Solo aplicable cuando la dimension es 1!!!*/
	public double derivatesPartialSecond(double y[], int direction1, int direction2) throws Exception
	{
		if(direction1 == direction2)
		{
			return this.derivatesPartialSecond(y, direction1);
		}
		double [] p1 = Util.vectorialRest(y, Util.scalarMultiply(Util.ei(y.length, 0), -2 * Util.CONSTANTE_3));
		double [] p2 = Util.vectorialRest(y, Util.scalarMultiply(Util.ei(y.length, 0), -Util.CONSTANTE_3));
		double [] p3 = Util.vectorialRest(y, Util.scalarMultiply(Util.ei(y.length, 0), Util.CONSTANTE_3));
		double [] p4 = Util.vectorialRest(y, Util.scalarMultiply(Util.ei(y.length, 0), 2 * Util.CONSTANTE_3));
		
		return (this.derivatesPartial(p1, 0)[0] - 8 * this.derivatesPartial(p2, 0)[0] + 8 * this.derivatesPartial(p3, 0)[0] - this.derivatesPartial(p4, 0)[0]) / ( 12*Util.CONSTANTE_3 ); 
	}
	
	public VariabledFunction getAsL2Operator()
	{
		String s = "";
		String prefix = "+";
		for(int k=0; k< this.dimension; k++)
		{
			prefix = k>0 ? "+" : "";
			s += prefix + "(" + this.getFunctionAt(k) + ")^2";
		}
		s = "sqrt(" + s + ")";
		return new VariabledFunction(s);
	}
	
	public double getNormLP(Interval i, double p) throws Exception
	{
		String s = "";
		String prefix = "+";
		for(int k=0; k< this.dimension; k++)
		{
			prefix = k>0 ? "+" : "";
			s += prefix + "(" + this.getFunctionAt(k) + ")^"+p;
		}
		s = "(" + s + ")^(-p)";
		return new VariabledFunction(s).integrates(i.getInf()+Util.EPSILON, i.getSup()-Util.EPSILON);
	}
	
	public double integrate(Interval i) throws Exception
	{
		VariabledFunction f = this.changeVariable("x", "(("+i.getSup()+")*x+(1-x)*("+i.getInf()+"))");
		return f.integrates(Util.EPSILON, 1-Util.EPSILON) * i.getLength();
		
	}
	public boolean belongsToLp(Interval i, double p) throws Exception
	{
		String s = "";
		String prefix = "+";
		System.out.println(this.getFunctionAt(0));
		for(int k=0; k< this.dimension; k++)
		{
			System.out.println(this.getFunctionAt(k));
			prefix = k>0 ? "+" : "";
			s += prefix + "(" + this.getFunctionAt(k) + ")^"+p;
		}
		s = "(" + s + ")^(1/"+p+")";
		System.out.println(s);
		double d=0;
		try
		{
			d = new VariabledFunction(s).integrates(i.getInf()+Util.EPSILON, i.getSup()-Util.EPSILON);
			return true;
		}
		catch (Exception e) {
			// TODO: handle exception
			return false;
		}
	}
	
	public boolean belongsToInfiniteLp(double p) throws Exception
	{
		String s = "";
		String prefix = "+";
		for(int k=0; k< this.dimension; k++)
		{
			prefix = k>0 ? "+" : "";
			s += prefix + "(" + this.getFunctionAt(k) + ")^"+p;
		}
		s = "(" + s + ")^(1/"+p+")";
		
		double d = new VariabledFunction(s).integrateInfinite();
		
		if(Double.isInfinite(d) || Double.isNaN(d))
		{
			return false;
		}
		return true;
	}
	
	public VariabledFunction scalarProduct(VariabledFunction f) throws Exception
	{
		String [] s = new String[this.getDimension()];
		String retorno = "";
		for(int k=0; k< this.parser.getDimension(); k++)
		{
			s[k] += "(" + f.getFunctionAt(k) + ")*(" + this.getFunctionAt(k) + ")";
		}
		retorno = s[0];
		for(int k=1; k< this.parser.getDimension(); k++)
		{
			retorno += " + " + s[k];
		}
		return new VariabledFunction(retorno);
	}
	
	public VariabledFunction changeVariable(String antVar, String formula)
	{
		String [] function = new String[this.dimension];
		
		for(int k=0; k< this.getDimension(); k++)
		{
			function[k] = this.getFunctionAt(k).replaceAll(antVar, formula);
		}
		return new VariabledFunction(function);
	}

	public double integrateSemiInfinite(double limit, boolean positiveInf) throws Exception
	{

		VariabledFunction ff = null;
		String function = null;
		double b = limit != 1 ? 1 : 0.3;
		b=1;
		if(positiveInf)
		{
			function = this.changeVariable("x", "(-x)").getFunctionAt(0);
			function = "("+function+")*(-1)";
			return (-1) * new VariabledFunction(function).integrateSemiInfinite((-1)*limit, false);
		}
		if(limit>0)
		{
			return this.integrateSemiInfinite(0, false) + this.integrates(0, limit);
		}
			if(limit != 0)
			{
				//x = limit/t-1
				//dx =  -limit/(t-1)^2 dt
				System.out.println("Aqui");
				function = this.changeVariable("x", "((-1)*("+limit+")/(x-1))").getFunctionAt(0);
				function = "(-1)*("+function+")*("+limit+")/((x-1)^2)";
				ff = new VariabledFunction(function);
				System.out.println("function " + ff.getFunctionAt(0));
			}
			else
			{
				//x = t/t-1
				//dx =  1/(t-1)^2 dt
				function = this.changeVariable("x", "(x/(x-1))").getFunctionAt(0);
				function = "("+function+")/((x-1)^2)";
				ff = new VariabledFunction(function);
				System.out.println("function " + ff.getFunctionAt(0));
			}

		return ff.integrates(Util.EPSILON, b-Util.EPSILON);
	}
	
	public double integrateInfinite() throws Exception
	{
		String function = this.changeVariable("x", "1/x").getFunctionAt(0);
		function = "("+function+")*(x^(-2))";
		
		VariabledFunction ff = new VariabledFunction(function);
		double d = this.integrates(-1+Util.EPSILON, 1-Util.EPSILON);
		
		if(Double.isNaN(d) || Double.isInfinite(d) || Math.abs(d)>Util.BIG_M)
		{
			return new Double("NaN");
		}
		double d2 = ff.integrates(-1+Util.EPSILON, 1-Util.EPSILON);
		if(Double.isNaN(d2) || Double.isInfinite(d2) || Math.abs(d2)>Util.BIG_M)
		{
			return new Double("NaN");
		}
		
		return d + d2;
	}
	

	public double integrates(Interval i) throws LPException
	{
		try {
			return this.integrates(i.getInf(), i.getSup());
		} catch (LPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw e;
		}
	}
	
	public double integrateDouble(Interval ix, Interval iy, int precision) throws Exception
	{
		return this.integrateDouble(ix.getInf(), ix.getSup(), iy.getInf(), iy.getSup(), precision);
	}
	
	public double integrateDouble(Interval ix, Interval iy) throws Exception
	{
		return this.integrateDouble(ix, iy, 1000);
	}
	
	public double integrates(double limit_inf, double limit_sup) throws LPException
	{
		double h0 = 0.001;

		double h = h0;
		double [] y = new double[this.dimension];
		double x = limit_inf;
		double xn = limit_sup;
		double tol = FourierSerie.TOL_INI;
		
		New65f r = new New65f(tol, h, this);
		y[0] = 0.0;

		h = h0;
		r.init();
		do
		{
			h = r.getH();
			if( h<1e-10)
			{
				System.out.println("Error in " + x + " H " + h);
				throw new LPException();
				//return new Double("NaN");
			}
		    
			h = (x+h> xn) ? xn - x : h;
			r.setH(h);
			try
			{
				y = r.evaluaPaso(x, y);
			}
			catch (LPException e) {
				// TODO: handle exception
				System.out.println("Error in " + x);
				throw e;
				
			}

			h = r.getH0();
			x = x + h;
		}
		while (x< xn);
		return  y[0];
	}
	

	public double integrateDouble2(double limit_inf, double limit_sup, double limit_infy, double limit_supy) throws Exception
	{
		double hy = (limit_supy-limit_infy)/10;
		double hx = (limit_supy-limit_inf)/10;
		
		double mul = hx*hy;
		
		double x = limit_inf;
		double y = limit_infy;
		double integrate = 0;

		String function = this.getFunctionAt(0);
		VariabledFunction integrater = null ;
			do
			{
				y+=hy;
				y = y-hy>limit_sup ? limit_sup-hy : y;
				integrater = new VariabledFunction(function.replaceAll("y", Double.toString(y)));
				integrate += integrater.integrates(limit_inf, limit_sup);
				
				//System.out.println("f en " + x+hx/2 + " " + y+hy/2 + " " + this.f(new double[]{x+hx/2, y+hy/2})[0]);
			}while (y< limit_supy);
			
		
		return integrate;
		
	}
	/*Solo aplicable cuando la dimension es 1!!!*/
	public double [][] hessian(double [] y) throws Exception
	{
		double [][] dReturn = new double[y.length][y.length];
		
		for(int j= 0; j< y.length; j++)
		{
			for(int k= 0; k< y.length; k++)
			{
				dReturn[j][k] = this.derivatesPartialSecond(y, k, j);
			}
		}
		return dReturn;
	}
	
	/* so funciona se f=f(x)*/
	public double getMin(double h, double begin, double end) throws Exception
	{
		 double min = Util.BIG_M;
		 double x = begin;
		 double f = end;
		 while(x < end)
		 {
			 f = f(x, new double[1])[0];
			 min = f< min ? f : min;
			 x = x + h;
		 }
		 return min;
	}
	/* so funciona se f=f(x)*/
	public double getMax(double h, double begin, double end) throws Exception
	{
		 double min = 0;
		 double x = begin;
		 double f = end;
		 while(x < end)
		 {
			 f = f(x, new double[1])[0];
			 min = f > min ? f : min;
			 x = x + h;
		 }
		 return min;
	}
	
	public boolean isExtendedFunction()
	{
		return false;
	}
	
	public FourierSerie init(int index, int size)
	{
		List<Coeficent> lista = new ArrayList<Coeficent>();
		Controlador controlador;
		
			controlador =  new Controlador(this.getFunctionAt(index), size);
		
		new Thread(controlador).start();
		while(controlador.isConnecting())
		{
			if(controlador.getCurrentTime() > Controlador.MAX_TIME)
			{
				controlador.setConnecting(false);
				controlador.setError(true);
			}
			try
			{
				Thread.currentThread().sleep(1000);
			}
			catch (Exception ex) 
			{
				ex.printStackTrace();
			}
		}
		
		return controlador.getfSerie();
	}
	
	public FourierSerie getFourierSeries(int size, Interval interval, int kind) throws LPException
	{
		FourierSerie fourierSerie = new FourierSerie(this, size, interval, kind);
		
		fourierSerie.formateFunctions();
		try {
			fourierSerie.calculateCoeficents();
		} catch (LPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw e;
		}
		
		return fourierSerie;
	}
	
	public VariabledFunction composition(String function) 
	{
		VariabledFunction f = new VariabledFunction(function);
		return this.composition(f);
	}
	public VariabledFunction composition(VariabledFunction f2) {
		String[] variables;
		switch(f2.dimension)
		{
		case 2:
			variables = new String[]{"x", "y"};
			break;
		case 3:
			variables = new String[]{"x", "y", "z"};
			break;
		case 4:
			variables = new String[]{"x", "y", "z", "t"};
			break;
		case 5:
			variables = new String[]{"x", "y", "z", "t", "w"};
			break;
		default:
			variables = new String[]{"x", "t"};
			break;
		}
		
		int k =0;
		VariabledFunction composition = null;
		try {
			composition = (VariabledFunction) clone();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for(String var: variables)
		{
			
			composition.getParser().composeAtVariable(var, f2.getParser().getConfig()[k].getExpression());
			k++;
		}
		return composition;
	}
	
	public Object clone(){
	        Object obj=null;
	        try{
	            obj=super.clone();
	        }catch(CloneNotSupportedException ex){
	        }
	        return obj;
	    }

	public int getDimension() {
		return dimension;
	}

	public void setDimension(int dimension) {
		this.dimension = dimension;
	}

	public VariabledFParser getParser() 
	{
		return this.parser;
	}
	public VariabledFParser getParserForX(double x) {
			return getParser();
	}

	public void setParser(VariabledFParser parser) {
		this.parser = parser;
	}

	public String[] getFuncion() {
		return funcion;
	}

	public void setFuncion(String[] funcion) {
		this.funcion = funcion;
	}
	
	public String toString()
	{
		return this.getFunctionAt(0) == null ? "" : this.getFunctionAt(0);
	}
	
	public List<PointRepresentable> getRepresentablePoints(Interval interval, double sizeStep) throws Exception
	{
		List<PointRepresentable> pointList = new ArrayList<PointRepresentable>();
		
		double x = interval.getInf();
		Double f = null;
		if(this.getDimension()==1)
		{
			do
			{
				f = new Double(f(x));
				if(f.isInfinite() || f.isNaN())
				{
					f = 0.0;
				}

				pointList.add(new PointRepresentable(x, f));
				x += sizeStep; 

			}while(x <= interval.getSup());
		}
		else
		{
			double [] ff;
			do
			{
				ff = f(x, new double[0]);
				
				pointList.add(new PointRepresentable(ff[0], ff[1]));
				x += sizeStep; 

			}while(x <= interval.getSup());
		}
		
		return pointList;
	}
	
	public double[] findZeros() throws Exception
	{
		return this.findZeros(-1000, 1000);
	}
	
	public double[] findZeros(double a, double b) throws Exception
	{
		ArrayList<Double> listaCeros = new ArrayList<Double>();
		double zero = 0;
		int k0 = 1;
		int zeroCount=0;
		double zeroAnt=0;
		Interval i = new Interval(a, b);
		for(int k=k0; k< b; k++)
		{
			zero = Math.abs(this.f(k))<Util.EPSILON ? this.f(k) : this.findZero(0, new double[]{k});
			if(zero != -30001) 
			{
				if(!listaCeros.contains(new Double(zero)) && (Math.abs(zeroAnt - zero) > Util.CONSTANTE_3 || Math.abs(zero) < Util.EPSILON) && i.belongsToMe(zero))
				{
					if(!this.listaContensNumber(listaCeros, zero))
					{
						listaCeros.add(zeroCount, new Double(zero));
						zeroAnt = zero;
						zeroCount++;
					}
				}
			}
		}
		
		zeroAnt=0;
		
		zeroCount = listaCeros.size();
		for(int k=k0; k>a; k--)
		{
			
			zero = Math.abs(this.f(k))<Util.EPSILON ? this.f(k) : this.findZero(0, new double[]{k});
			if(zero != -30001) 
			{
				if(!listaCeros.contains(new Double(zero)) && (Math.abs(zeroAnt - zero) > Util.CONSTANTE_3 || Math.abs(zero) < Util.EPSILON) && i.belongsToMe(zero) )
				{
					if(!this.listaContensNumber(listaCeros, zero))
					{
						listaCeros.add(zeroCount, new Double(zero));
						zeroAnt = zero;
						zeroCount++;
					}
				}
			}
		}
		
		
		double [] zeros = new double[listaCeros.size()];
		k0 = 0;
		for(double d: listaCeros)
		{
			zeros[k0] = d;
			k0++;
		}
		return zeros;
	}
	
	public boolean pointIsLocalMaximum(double p) throws Exception
	{
		return this.f(p-Util.CONSTANTE_5) < this.f(p) &&  this.f(p+Util.CONSTANTE_5) < this.f(p); 
	}
	
	public boolean pointIsLocalMinimum(double p) throws Exception
	{
		return this.f(p-Util.CONSTANTE_5) > this.f(p) &&  this.f(p+Util.CONSTANTE_5) > this.f(p); 
	}
	
	public boolean pointIsChairPoint(double p) throws Exception
	{
		return this.f(p-Util.CONSTANTE_5) > this.f(p) &&  this.f(p+Util.CONSTANTE_5) < this.f(p) || this.f(p-Util.CONSTANTE_5) < this.f(p) &&  this.f(p+Util.CONSTANTE_5) > this.f(p); 
	}
	
	public String getPointDescription(double d) throws Exception
	{

		
		if(this.pointIsLocalMaximum(d))
		{
			return "Local Maximum";
		}
		
		if(this.pointIsLocalMinimum(d))
		{
			return "Local Minimum";
		}
		
		if(this.pointIsChairPoint(d))
		{
			return "Chair Point";
		}
		
		return "Unknown";
	}
	
	public boolean listaContensNumber(List<Double> lista, double d)
	{
		for(Double dd: lista)
		{
			if(Math.abs(dd-d)<Util.EPSILON)
				return true;
		}
		
		return false;
	}

	

	public double findZero() throws Exception
	{
		double zero = -30001;
		int k0 = this.derivates(0)< Util.EPSILON ? this.derivates(1)< Util.EPSILON ? -1 :1 : 0;
		
		for(int k=k0; k< 100000; k++)
		{
			zero = this.findZero(0, new double[]{k});
			if(zero != -30001) return zero;
		}
		
		for(int k=k0; k> 100000; k--)
		{
			zero = this.findZero(0, new double[]{k});
			if(zero != -30001) return zero;
		}
		
	
		return -30001;
	}
	
	/*Search Zero using the Newton method*/	
	public double findZero(int index, double[] x) throws Exception
	{
		double xn = x[0];		
		int n=0;
		double derivate = 0;
		do
		{
			x[0]=xn;
			for(int k=0; k< x.length; k++)
			{
				derivate = this.derivates(x[0]);
				xn = derivate != 0 ? x[0] - this.f(x)[0]/derivate : -30001;
			}
			n++;
			if(n>20 || xn == -30001)
			{
				return -30001;
			}
		}while(Math.abs(this.f(xn))>Util.EPSILON);
		return xn;
	}
	
	/*Search Zero in the derivate using the Newton method*/	
	public double findZeroInDerivate(int index, double[] x) throws Exception
	{
		double xn = x[0];		
		int n=0;
		double derivate = 0;
		do
		{
			x[0]=xn;
			for(int k=0; k< x.length; k++)
			{
				derivate = this.derivatesSecond(new double[]{x[0]});
				xn = derivate != 0 ? x[0] - (this.derivates(x[0]))/derivate : -30001;
			}
			n++;
			if(n>20  || xn == -30001)
			{
				return -30001;
			}
		}while(Math.abs(this.f(xn))>Util.EPSILON);
		return xn;
	}
	
	/*Points where derivates is cero*/
	public double [] getCriticPoints() throws Exception
	{
		return this.differenciate("x").findZeros();
	}
	
	/*Points where derivates is cero*/
	public double [] getCriticPoints(String variable) throws Exception
	{
		return this.differenciate(variable).findZeros();
	}
	
	public int clasifficCriticPoint(double criticPoint) throws Exception
	{
		double d = this.derivatesSecond(new double[]{criticPoint});
		return d>0 ? Util.IS_MINIMUM : d<0 ? Util.IS_MAXIMUM : Util.IS_INFLECTED;
	}
	
	public boolean isMaximum(double d) throws Exception
	{
		return this.derivatesSecond(new double[]{d}) < 0;
	}
	
	public void simplify()
	{
		this.parser.simplify();
	}
	
	public double integrateDouble(double limit_inf, double limit_sup, double limit_infy, double limit_supy) throws Exception
	{
		return integrateDouble(limit_inf, limit_sup, limit_infy, limit_supy, 1000);
	}
	
	public double integrateDouble(double limit_inf, double limit_sup, double limit_infy, double limit_supy, int precision) throws Exception
	{
		double hx = (limit_sup-limit_inf)/precision;
		double hy = (limit_supy-limit_infy)/precision;
		double mul = hx*hy;
		double x = limit_inf;
		double y = limit_infy;
		double integrate = 0;
		do
		{
			x+=hx;
			x = x-hx>limit_sup ? limit_sup-hx : x;
			//System.out.println("x " + x);
			y = limit_infy;
			
			do
			{
				y+=hy;
				y = y-hy>limit_sup ? limit_sup-hy : y;
				integrate += (mul)*(f(new double[]{x, y+hy})[0] + f(new double[]{x+hx, y})[0] 
                                       + f(new double[]{x+hx, y+hy})[0] + f(new double[]{x, y})[0])/4;
				//System.out.println("f en " + x+hx/2 + " " + y+hy/2 + " " + this.f(new double[]{x+hx/2, y+hy/2})[0]);
			}while (y< limit_supy);
			
		}while (x< limit_sup);
		
		return integrate;
		
	}
	
	public VariabledFunction sum(VariabledFunction f)
	{
		String [] s = new String[this.getDimension()];
		if(f.getDimension() != this.getDimension())
		{
			return null;
		}
		for(int k=0; k< this.getDimension(); k++)
		{
			s[k] = "(" + this.getFunctionAt(k) + ") + (" + f.getFunctionAt(k) + ")";
		}
		return new VariabledFunction(s);
	}
	
	
	public VariabledFunction evaluateAtVariable(String variable, double value)
	{
		String [] s = new String[this.getDimension()];
		for(int k=0; k< this.dimension; k++)
		{
			s[k] = this.getFunctionAt(k).replaceAll(variable, Double.toString(value));
		}
		return new VariabledFunction(s);
	}
}
