/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.fourier;

import java.util.ArrayList;
import java.util.List;

import com.carlos.math.functions.ExtendedFucntion;
import com.carlos.math.functions.Interval;
import com.carlos.math.functions.VariabledFunction;
import com.numerico.algebralineal.Util;

public class BaseCos 
{
	private int size;
	private List<String> cosSerie;
	private Interval interval;
	
	public BaseCos(int size, Interval i, int kind) 
	{
		this.size = size;
		cosSerie = new ArrayList<String>();
		interval = i;
		
		double divide = 2;
		if(kind != FourierSerie.KIND_FOURIER)
		{
			divide = 1;
		}
		//divide = 2;
		cosSerie.add("0.5");
		for(int k=1; k<= size; k++)
		{
			cosSerie.add("cos("+ Util.PI + "*" + k + "x/" + (interval.getLength()/divide) + ")");
		}
	}

	public BaseCos(int size, int kind) 
	{
		this(size, new Interval(-Util.PI, Util.PI), kind);
	}
	
	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public List<String> getCosSerie() {
		return cosSerie;
	}

	public void setCosSerie(List<String> cosSerie) {
		this.cosSerie = cosSerie;
	}
	public List<VariabledFunction> getListaXBaseOf(VariabledFunction f)
	{
		List<VariabledFunction> lista = new ArrayList<VariabledFunction>();
		
		if(!f.isExtendedFunction())
		{
			for(String ei: cosSerie)
			{	
				lista.add(new VariabledFunction("(" + f.getFunctionAt(0) + ")*(" + ei + ")"));
			}
		}
		else
		{
			ExtendedFucntion ef = (ExtendedFucntion)f;
			List<VariabledFunction> listaFunctiones;
			for(String ei: cosSerie)
			{
				listaFunctiones = new ArrayList<VariabledFunction>();
				//String [] efes = new String[ef.getFunctionsLength()];
				
				for(VariabledFunction function : ef.getFunctions())
				{	
					listaFunctiones.add(new VariabledFunction("(" + function.getFunctionAt(0) + ")*(" + ei + ")"));
					//System.out.println("Adding " + "(" + function.getFunctionAt(0) + ")*(" + ei + ")");
				}
				//System.out.println("");
				lista.add(new ExtendedFucntion(listaFunctiones, ef.getNodes()));
			}
		}
		return lista;
	}
	
	public static List<VariabledFunction> getBase(int size, Interval i, int kind)
	{
		List<VariabledFunction> lista = new ArrayList<VariabledFunction>();
		
		double divide = 2;
		if(kind != FourierSerie.KIND_FOURIER)
		{
			divide = 1;
		}
		divide = 2;
		lista.add(new VariabledFunction("1"));
		for(int k=1; k<= size; k++)
		{
			lista.add(new VariabledFunction("cos("+ Util.PI + "*" + k + "x/" + (i.getLength()/divide) + ")"));
		}
		
		return lista;
	}
	
	public static List<VariabledFunction> getBase(int size, int kind)
	{
		return BaseCos.getBase(size, new Interval(-Util.PI, Util.PI), kind);
	}
	
}
