/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.fourier;

import de.jtem.mathExpr.UnknownDefinitionException;

import com.carlos.math.functions.ExtendedFucntion;
import com.carlos.math.functions.Interval;
import com.carlos.math.functions.VariabledFunction;
import com.numerico.algebralineal.Util;



public class Controlador implements Runnable
{
	private FourierSerie fSerie;
	private boolean connecting, error;
	public static final int MAX_TIME = 60;
	private long time;
	
	public static void main(String [] args) throws Exception
	{
		ExtendedFucntion ef = new ExtendedFucntion(new String[]{"x"}, new double[]{-1, 1});
		FourierSerie fourierSeries = ef.getFourierSeries(3, new Interval(-1, 1), FourierSerie.KIND_FOURIER);
		System.out.println("**************************");
		System.out.println("**************************");
		for(VariabledFunction f :fourierSeries.getSeriesAsFunctions())
		{
			System.out.println(f.getFunctionAt(0));
		}
		System.out.println("**************************");	
		
		//VariabledFunction f2 = new VariabledFunction("x^2");
		//FourierSerie fu = new FourierSerie(f2, 1);
		//double d = f2.getFourierSeries(500, new Interval(-Util.PI, Util.PI), FourierSerie.KIND_FOURIER).evaluateAt(1);
		
		System.out.println("valor " + fourierSeries.evaluateAt(-1));
		
	}
	
	public Controlador(String function, int size) {
		fSerie = new FourierSerie(new VariabledFunction(function));
		fSerie.setSize(size);
		init();
		time = System.currentTimeMillis();
		connecting = true;
		error = false;
	}
	
	public Controlador(ExtendedFucntion funcion, int size) {
		
		fSerie = new FourierSerie(funcion);
		fSerie.setSize(size);
		init();
		time = System.currentTimeMillis();
		connecting = true;
		error = false;
	}
	
	public void run() 
	{

		try
		{
			initCalcules();
		}
		catch (Exception e) {
			this.error = true;
			connecting = false;
		}
	}
	
	private void init()
	{
		fSerie.formateFunctions();
	}
	
	private void initCalcules() throws Exception
	{
		try
		{
			fSerie.calculateCoeficents();
		}
		catch (UnknownDefinitionException e) {
			error = true;
			throw e;
		}
		
		connecting = false;
	}
	

	public long getTime() {
		return time;
	}
	
	public float getCurrentTime()
	{
		return (float)(System.currentTimeMillis() - time) /1000;		
	}
	public FourierSerie getfSerie() {
		return fSerie;
	}
	public void setfSerie(FourierSerie fSerie) {
		this.fSerie = fSerie;
	}
	public boolean isConnecting() {
		return connecting;
	}
	public void setConnecting(boolean connecting) {
		this.connecting = connecting;
	}
	public boolean isError() {
		return error;
	}
	public void setError(boolean error) {
		this.error = error;
	}
	public void setTime(long time) {
		this.time = time;
	}

	
}
