/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.rkuta;

import java.lang.reflect.Constructor;

import com.carlos.math.functions.LPException;
import com.carlos.math.functions.VariabledFunction;
import com.numerico.algebralineal.Util;

public abstract class RungeKutta {

	protected int nstep, order;
	protected int rejected;
	protected int neval, dimension;
	protected double tolerancia, h, x0, h0, est;
	protected VariabledFunction f;
	protected int etapa = 0;
	protected boolean iflag, finalStep;
	protected VariabledFunction funcionExacta;
	protected Solucion solucion;
	protected String methodName;
	protected double[][] k;
	protected double[] w;
	
	public RungeKutta()
	{
		order = 0;
		this.nstep = 0;
		this.rejected = 0;
		this.neval = 0;
		this.iflag = true;
		this.finalStep = false;
		this.est = 0;
		this.methodName = "Rung-Kutta";
	}
	
	protected void calculaSolucion(double x, double [] y)
	{
		this.solucion = new Solucion(x, y, this.getH(), this.funcionExacta, this);
	}
	
	
	
	public void updateInIndex(int index, double x) 
	{
		//System.out.println(SimplexUtil.printVector2("w", f.f(x, w)));
		w = f.f(x, w);
		
		this.neval++;
		this.etapa++;
		for(int j=0; j< f.getDimension(); j++)
		{
			k[index][j] = w[j];
		}		
	}
	
	public void avanzaUnPaso(double x, double[] y) throws LPException {
		// TODO Auto-generated method stub
		nstep++;
		etapa = 0;
		double d = Util.normaLp(y, 2);
		//System.out.println("VALEU"+y[0]);
		if(Double.isNaN(d)|| Double.isInfinite(d))
		{
			
			LPException e = new LPException("Error LP 2 at " + x + " function " + this.getF().getFunctionAt(0));
			throw e;
		}
		calculaSolucion(x + h, y);
		iflag = true;
	}
	public void recalculaH(double newH) throws LPException
	{
		h0 = h;
		h = newH;
		/*
		if(h<Util.EPSILON && !isFinalStep())
		{
			throw new LPException();
		}
		*/
	}
	
	public abstract void init();
	public abstract double [] evaluaPaso(double x, double [] y) throws Exception;
	
	public int getNstep() 
	{
		return nstep;
	}
	public int getRejected() {
		return rejected;
	}
	
	public int getNEval()
	{
		return neval;
	}

	public int getDimension() {
		return dimension;
	}

	public void setDimension(int dimension) {
		this.dimension = dimension;
	}

	public double getTolerancia() {
		return tolerancia;
	}

	public void setTolerancia(double tolerancia) {
		this.tolerancia = tolerancia;
	}

	public double getH() {
		
		return h;
	}

	public void setH(double h) {
		this.h = h;
	}

	public boolean isIflag() {
		return iflag;
	}

	public void setIflag(boolean iflag) {
		this.iflag = iflag;
	}

	public int getNeval() {
		return neval;
	}

	public void setFuncionExacta(VariabledFunction funcionExacta) {
		this.funcionExacta = funcionExacta;
	}

	public double getX0() {
		return x0;
	}

	public void setX0(double x0) {
		this.x0 = x0;
	}
	
	
	public Solucion getSolucion() {
		return solucion;
	}

	public String getMethodName() {
		return methodName;
	}

	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}

	public String printStatus(double x) throws Exception
	{
		String status = " 		- " + this.getMethodName() + " - \n"; 
		status += "*************** STATUS en x = " + x+ " *************\n" +
		"\tPasos " + getNstep() + "\n" +
		"\tEvaluaciones de la funcion " + getNEval() + " (x*" + dimension + ")\n" +
		"\tRechazos " + getRejected() + "\n" +
		"\tLong. ultimo paso " + getH() + "\n" +
		"\tUltimo Error " + solucion.getMaxError()  + "\n" + 
		"\tFunciones "  + "\n";
		int j=1;
		for(int k=0; k< f.getDimension(); k++)
		{
			status += "\ty" + j + " = " + f.getFunctionAt(k)  + "\n";
			j++;
		}
		System.out.println("");
		System.out.println("Function Exacta :::");
		for(int k=0; k< f.getDimension(); k++)
		{
			status += "\tye" + k + " = " + this.funcionExacta.getFunctionAt(k)  + "\n";
			j++;
		}
		System.out.println("");
		status += "\tSolucion en x " + getSolucion().getT() + "\n";
		j=1;
		for(double d: solucion.getY())
		{
			status += "\ty[" + j +"] = " + d + "\n";	
			j++;
		}
		
		if(funcionExacta != null)
		{
			status += "\tExacta \n";
			for(int k = 0; k< this.funcionExacta.getDimension(); k++)
			{
				System.out.println("Dimension " + k);
				if(!funcionExacta.isNullFunction())
				{
					status += funcionExacta.getFunctionAt(k) + "\t " + funcionExacta.f(new double[]{x})[k] + " \n";
				}
				else
				{
					status += "Funcion exata es null";
				}
			}
			
		}
		
		status += "\tSolucion exacta \n";
		status += "\t" + solucion.toString() + " \n";
		status += "*********************************************";
		
		return status;
	}

	public VariabledFunction getF() {
		return f;
	}

	public void setF(VariabledFunction f) {
		this.f = f;
		this.dimension = f.getDimension();
	}

	public double getH0() {
		return h0;
	}

	public void setH0(double h0) {
		this.h0 = h0;
	}

	public int getOrder() {
		return order;
	}

	public void setOrder(int order) {
		this.order = order;
	}
	
	public static String getMethodDescription(String methodName)
	{
		String className = "com.carlos.math.numerico.rkuta." + methodName;
		try
		{
			Class clazz = Class.forName(className);
			Constructor cons = clazz.getConstructor(double.class, VariabledFunction.class);
		
			RungeKutta r = (RungeKutta) cons.newInstance(0, new VariabledFunction(new String[]{}));
			return r.getMethodName();
		}
		catch (Exception e) {
			return "";
		}
	}

	public VariabledFunction getFuncionExacta() {
		return funcionExacta;
	}

	public boolean isFinalStep() {
		return finalStep;
	}

	public void setFinalStep(boolean finalStep) {
		this.finalStep = finalStep;
	}
	
	
}
