/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.util;

import de.jtem.mathExpr.ExpressionConfiguration;
import de.jtem.mathExpr.UnknownDefinitionException;
import de.jtem.mathExpr.evaluator.realEvaluator.Real;
import de.jtem.mathExpr.evaluator.realEvaluator.RealType;
import de.jtem.mathExpr.parser.Symbol;
import de.jtem.mathExpr.parser.javacc.ParseException;

public class FParser {

	private ExpressionConfiguration config;
	private Real x, y, z, t, w, m;
	private Real [] real;
	
	public FParser()
	{
		this("x");
	}
	public FParser(String function)
	{
		if(function == null || function.length()<1)
			return;
		config = new ExpressionConfiguration(RealType.TYPE);

		config.defineFunction("f", new String[] {"x"}, "sin(x)");
		config.defineFunction("g", new String[] {"x"}, "cos(x)");
		/*config.defineFunction("tan", new String[] {"x"}, "(sin(x))/(cos(x))");*/
		config.defineFunction("cotan", new String[] {"x"}, "(cos(x))/(sin(x))");
		config.defineFunction("cosec", new String[] {"x"}, "(1)/(sin(x))");
		config.defineFunction("sec", new String[] {"x"}, "(1)/(cos(x))");
		config.defineFunction("cosh", new String[] {"x"}, "(e^(x) - e^(-x))/(2)");
		config.defineFunction("sinh", new String[] {"x"}, "(e^(x) + e^(-x))/(2)");
		config.defineFunction("tanh", new String[] {"x"}, "(sinh(x))/(cosh(x))");
		config.defineFunction("cotanh", new String[] {"x"}, "(cosh(x))/(sinh(x))");
		config.defineFunction("cotanh", new String[] {"x", "x"}, "y*sin(x)");
		
		w = new Real();
		x = new Real();
		y = new Real();
		z = new Real();
		t = new Real();
		m = new Real();
		
		config.defineVariable("x", x, RealType.TYPE);
		config.defineVariable("y", y, RealType.TYPE);
		config.defineVariable("z", z, RealType.TYPE);
		config.defineVariable("t", t, RealType.TYPE);
		config.defineVariable("m", m, RealType.TYPE);
		config.defineVariable("w", w, RealType.TYPE);

		config.setExpression(function);
		
	}
	public void addVariable(String varName)
	{
		config.defineVariable(varName, new Real(), RealType.TYPE);
	}
	public String differenciate(String variable)
	{
		return config.differentiate(variable).toString();
	}
	public double evaluate(double d, String function)
	{
		config.setExpression(function);
		x.assign(d);
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	
	public String composeAtVariable(String variable, FParser parser)
	{
		ExpressionConfiguration e = new ExpressionConfiguration(RealType.TYPE);
		Symbol s = new Symbol(variable);
		e.setExpression(config.getExpression().replaceSymbol(s, parser.getConfig().getExpression()));
		
		System.out.println(parser.getConfig().getExpression() + " En la compo " + e.getExpression().toString());
		return e.getExpression().toString();
	}
	public double evaluateArray(double [] d)
	{
		real = new Real[d.length];
		
		for(int j=0; j< real.length; j++)
		{
			if(j == 0)
			{
				t.assign(d[j]);
			}
			else
			{
				real[j] = new Real();
				config.defineVariable("y" + (j-1), real[j], RealType.TYPE);
				real[j].assign(d[j]);
			}
			
		}
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	
	public double evaluate(double [] d) throws Exception
	{
		switch (d.length)
		{
		case 1:
			return evaluate(d[0]);
		case 2:
			return evaluate(d[0], d[1]);
		case 3:
			return evaluate(d[0], d[1], d[2]);	
		case 4:
			return evaluate(d[0], d[1], d[2], d[3]);
		case 5:
			return evaluate(d[0], d[1], d[2], d[3], d[4]);
		case 6:
			return evaluate(d[0], d[1], d[2], d[3], d[4], d[5]);
		default:
			return evaluateArray(d);
		}	
	}

	public double evaluate(double x, double [] y) throws Exception
	{
		double temp [] = new double[y.length +1];
		for(int k=0; k< temp.length; k++)
		{
			temp [k] = (k != 0) ? y[k-1] : x;
		}
		
		return this.evaluate(temp);
	}
	
	
	public double evaluate(double d) throws Exception
	{
		x.assign(d);
		t.assign(d);
		try
		{
			return (new Real((Real)config.evaluateExpression())).getValue();
		}
		catch (Exception e) {
			throw e;
		}
	}
	
	
	
	public double evaluate(double d, double d2) throws Exception 
	{
		if(config == null)
			return 0;
		x.assign(d);
		t.assign(d);
		y.assign(d2);
		try
		{
			return (new Real((Real)config.evaluateExpression())).getValue();
		}
		catch (Exception e) {
			throw e;
		}
		
	}
	
	public double evaluate(double d, double d2, double d3)
	{
		t.assign(d);
		x.assign(d2);
		y.assign(d3);
		
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	public double evaluate(double d, double d2, double d3, double d4)
	{
		t.assign(d);
		x.assign(d3);
		y.assign(d3);
		z.assign(d4);
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	
	public double evaluate(double d, double d2, double d3, double d4, double d5)
	{
		t.assign(d);
		x.assign(d3);
		y.assign(d3);
		z.assign(d4);
		w.assign(d5);
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	public double evaluate(double d, double d2, double d3, double d4, double d5, double d6)
	{
		t.assign(d);
		x.assign(d3);
		y.assign(d3);
		z.assign(d4);
		w.assign(d5);
		m.assign(d6);
		return (new Real((Real)config.evaluateExpression())).getValue();
	}
	public static void main(String args[]) throws Exception
	{
		String funcion = "tanh(x)-y";
		FParser parser = new FParser(funcion);
		System.out.println(parser.evaluate(2.7181, 8));
	}

	public ExpressionConfiguration getConfig() {
		return config;
	}
}
