/*
    Copyright 2015 Carlos Toro Peas, Madrid, Espaa 
    This file is part of Android Integral Calculator. 
    Inherted from MaximaOnAndroid project whose
    Copyright 2012, Yasuaki Honda (yasuaki.honda@gmail.com)
    
    Android Integral Calculator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Android Integral Calculator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.carlos.math.numerico.util;

import java.util.ArrayList;
import java.util.List;

import com.carlos.math.functions.VariabledFunction;

import de.jtem.mathExpr.ExpressionConfiguration;
import de.jtem.mathExpr.evaluator.realEvaluator.Real;
import de.jtem.mathExpr.evaluator.realEvaluator.RealType;
import de.jtem.mathExpr.parser.Expression;
import de.jtem.mathExpr.parser.Symbol;

public class VariabledFParser 
{
	private de.jtem.mathExpr.ExpressionConfiguration config[];
	private List<List<RealVariable>> listaVariables;
	private int dimension;
	
	public VariabledFParser(String funcion)
	{
		this(1);
		config[0].setExpression(funcion);
	}
	public VariabledFParser(int dimension)
	{
		//this.listaVariables = new ArrayList<RealVariable>();
		
		this.listaVariables = new ArrayList<List<RealVariable>>();
		this.dimension = dimension;
		config = new ExpressionConfiguration[dimension];
		for(int k= 0; k< dimension; k++)
		{
			this.listaVariables.add(new ArrayList<RealVariable>());
			config[k] = new ExpressionConfiguration(RealType.TYPE);	
			
		}
	}
	
	public void addVariable(String variableName, int index)
	{
		List<RealVariable> l = this.listaVariables.get(index);
		l.add(new RealVariable(config[index], variableName));
		//this.listaVariables.add(new RealVariable(variableName, new Real(0), RealType.TYPE));
		config[index].defineVariable(variableName, new Real(), RealType.TYPE);
	}
	
	public boolean variableExists(String varName)
	{
		for(RealVariable v: this.listaVariables.get(0))
		{
			if(v.getName().equals(varName))
				return true;			
		}
		return false;
	}
	public void defineDefaultFunctions(int index)
	{
		config[index].defineFunction("abs", new String[] {"x"}, "sqrt(x*x)");
		config[index].defineFunction("sen", new String[] {"x"}, "sin(x)");
		config[index].defineFunction("log10", new String[] {"x"}, "log(x)/log(10)");
		
		config[index].defineFunction("sec", new String[] {"x"}, "1/(cos(x))");
		config[index].defineFunction("cosec", new String[] {"x"}, "1/(sin(x))");
		config[index].defineFunction("csc", new String[] {"x"}, "1/(sin(x))");
		config[index].defineFunction("cotan", new String[] {"x"}, "1/(tan(x))");
		config[index].defineFunction("cot", new String[] {"x"}, "1/(tan(x))");
		
		config[index].defineFunction("sinh", new String[] {"x"}, "(e^(x)-e^(-x))/2");
		config[index].defineFunction("senh", new String[] {"x"}, "sinh(x)");
		config[index].defineFunction("cosh", new String[] {"x"}, "(e^(x)+e^(-x))/2");
		config[index].defineFunction("tanh", new String[] {"x"}, "cosh(x)/sinh(x)");
		config[index].defineFunction("cotanh", new String[] {"x"}, "sinh(x)/cosh(x)");
		config[index].defineFunction("sech", new String[] {"x"}, "1/cosh(x)");
		config[index].defineFunction("cosech", new String[] {"x"}, "1/sinh(x)");

		
		config[index].defineFunction("asin", new String[] {"x"}, "atan(x/(sqrt(1-x^2)))");
		config[index].defineFunction("asenh", new String[] {"x"}, "asin(x)");
		config[index].defineFunction("acos", new String[] {"x"}, "pi/2 + atan(-x/(sqrt(1-x^2)))");
		
		config[index].defineFunction("asinh", new String[] {"x"}, "log(x+sqrt(x^2+1))");
		config[index].defineFunction("acosh", new String[] {"x"}, "log(x+sqrt(x^2-1))");
		config[index].defineFunction("atanh", new String[] {"x"}, "log(sqrt(1+x))-log(sqrt(1-x))");
		
		
		config[index].defineFunction("arcsin", new String[] {"x"}, "asin(x)");
		config[index].defineFunction("arcsen", new String[] {"x"}, "arcsin(x)");
		config[index].defineFunction("arccos", new String[] {"x"}, "acos(x)");
		config[index].defineFunction("arctan", new String[] {"x"}, "atan(x)");

		config[index].defineFunction("sech", new String[] {"x"}, "1/cosh(x)");
		config[index].defineFunction("csch", new String[] {"x"}, "1/sinh(x)");
		config[index].defineFunction("coth", new String[] {"x"}, "1/tanh(x)");
		
		config[index].defineFunction("asec", new String[] {"x"}, "acos(1/x)");
		config[index].defineFunction("acsc", new String[] {"x"}, "asin(1/x)");
		config[index].defineFunction("acot", new String[] {"x"}, "atan(1/x)");
		
		
		config[index].defineFunction("asech", new String[] {"x"}, "acosh(1/x)");
		config[index].defineFunction("acsch", new String[] {"x"}, "asinh(1/x)");
		config[index].defineFunction("acoth", new String[] {"x"}, "atanh(1/x)");
		/*Por definir
		config[index].defineFunction("acosec", new String[] {"x"}, "atan(x/(sqrt(1-x^2)))");
		config[index].defineFunction("asec", new String[] {"x"}, "pi/2 + atan(-x/(sqrt(1-x^2)))");
		config[index].defineFunction("acotan", new String[] {"x"}, "pi/2 + atan(-x/(sqrt(1-x^2)))");
		
		config[index].defineFunction("arccosec", new String[] {"x"}, "acosec(x)");
		config[index].defineFunction("arccosec", new String[] {"x"}, "asec(x)");
		config[index].defineFunction("arccotan", new String[] {"x"}, "acotan(x)");
		*/
		
		config[index].setExpression("abs(x)");
		//config[index].setExpression("asin(x)");0,78539816339744830961566084581988
		
	}
	public void addFunction(String function, int index)
	{
		config[index] = new ExpressionConfiguration(RealType.TYPE);
		this.defineDefaultFunctions(index);
		config[index].setExpression(function);
	}
	
	
	
	public void addFunction(String function)
	{
		ExpressionConfiguration[] config2 = new ExpressionConfiguration[config.length+1];
		for(int k=0; k< config.length; k++)
		{
			config2[k] = config[k];
		}
		config2[config.length] = new ExpressionConfiguration(RealType.TYPE);
		config2[config.length].setExpression(function);
		this.config = config2;
	}
	
	public String getFunctionAt(int index)
	{
		//return this.config[index].getExpression().toString();
		return config != null && config.length >= index + 1 && this.config[index].getExpression() != null ? this.config[index].getExpression().toString() : null;
	}
	
	public double[] evaluate(double [] d)
	{
		
		if(d.length != listaVariables.size())
		{
			//throw new Exception();
		}
		int k=0;
		for(List<RealVariable> lista : this.listaVariables )
		{
			k=0;
			for(RealVariable r : lista)
			{
			
				if(k>d.length-1)
				{
					r.getValue().assign(d[d.length-1]);
				}
				else
				{
					r.getValue().assign(d[k]);
				}
				k++;
			}
		}

		//System.out.println("Dimenion "+dimension + " " + this.getConfig()[0].getExpression() );

		
		double [] retorno = new double[dimension];
		for(int j= 0; j< dimension; j++)
		{
			//System.out.println("Evaluando " + config[j].getExpression());
			//System.out.println(SimplexUtil.printVector2("EN  ", d));
			retorno[j] = (new Real((Real)config[j].evaluateExpression())).getValue();
			//System.out.println("torno " + retorno[j] );
			
		}
		//System.out.println(SimplexUtil.printVector2("Salio ", retorno));
		//System.out.println("");
		return retorno;
	}
	public String printVariables()
	{
		String s = "";
		for(List<RealVariable> lista : this.listaVariables )
		{
			for(RealVariable r : lista)
			{
				s += r.getName() +"="+r.getValue()+", ";
			}
		}
		return s;
	}
	
	public VariabledFunction differenciate(String variable)
	{
		String [] s = new String [this.config.length];
		int k =0;
		for(ExpressionConfiguration e: config)
		{
			//e.getExpression().differentiate(arg0)
			s[k] = e.differentiate(variable).toString();
			k++;
		}
		return new VariabledFunction(s);
	}
	
	public void composeAtVariable2(String variable, VariabledFParser parser)
	{
		ExpressionConfiguration [] e = new ExpressionConfiguration[dimension];
		Symbol s = new Symbol(variable);
		for(int j= 0; j< dimension; j++)
		{
			e[j] = new ExpressionConfiguration(RealType.TYPE);
			for(int i =0; i< parser.getDimension(); i++)
			{
				e[j].setExpression(config[j].getExpression().replaceSymbol(s, parser.getConfig()[i].getExpression()));
				System.out.println(parser.getConfig()[j].getExpression() + " En la compo " + e[j].getExpression().toString());
			}
		}
	}

	
	public void composeAtVariable(String variable, Expression expresion)
	{
		Symbol s = new Symbol(variable);
		
		int k = 0;
		for(ExpressionConfiguration ex : this.config)
		{
			ex.setExpression(ex.replaceSymbol(s, expresion));
			k++;
		}
	}
	
	public boolean isCero()
	{
		boolean isCero = false;
		for(int k=0; k< this.dimension; k++ )
		{
			isCero = this.getFunctionAt(k).equals("0.0");
			if(isCero) return true;
		}
		return false;
	}
	
	
	public ExpressionConfiguration [] getConfig() {
		return config;
	}

	public void setConfig(ExpressionConfiguration [] config) {
		this.config = config;
	}

	public List<RealVariable> getListaVariables(int index) {
		return listaVariables.get(index);
	}

	public void setListaVariables(List<RealVariable> listaVariables, int index) {
		this.listaVariables.add(index, listaVariables);
		this.listaVariables.remove(index+1);
	}

	public int getDimension() {
		return dimension;
	}

	public void setDimension(int dimension) {
		this.dimension = dimension;
	}
	
	public Object clone()
	{
		VariabledFParser p = new VariabledFParser(dimension);
		for(int k =0; k< dimension; k++)
		{
			p.addConfiguration(config[k], k);
		}
		return p;
	}
	public void addConfiguration(ExpressionConfiguration c, int k)
	{
		this.config[k] = c;
	}
	
	public void simplify()
	{
		for(ExpressionConfiguration e: this.config)
		{
			e.simplify();
		}
	}
}
