package com.numerico.algebralineal;




public class Util 
{

	public static double PI = 3.1415926535897932384626433832795;
	public static double EPSILON = 1e-12;
	public static double CONSTANTE_5 = 1e-5;
	public static double CONSTANTE_3 = 1e-3;
	public static double BIG_M = 1e15;
	public static int IS_MAXIMUM = 1;
	public static int IS_MINIMUM = -1;
	public static int IS_INFLECTED = 0;
	public static int ITERATIONS_LIMIT = 100000;
	
	public static double[][] transporMatrix(double [][] matrix)
	{
		int n = matrix.length;
		int m = matrix[0].length;
		
		double [][]a = new double[m][n];
		
		for(int j=0; j< n; j++)
		{
			for(int k=0; k< m; k++)
			{
				a[k][j] = matrix[j][k];
			}
		}
		
		return a;
	}
	
	public static double[] zeros(int length)
	{
		double [] d = new double[length-1];
		for(int j =0; j< d.length; j++)
		{
			d[j] = 0;
		}
		return d;
	}
	
	/*
	 * Returns a (0, 0, ..., 1, 0)
	 * vector
	 */
	public static double[] ei(int length, int i)
	{
		double [] d = new double[length];
		for(int j =0; j< d.length; j++)
		{
			d[j] = j != (i) ? 0 : 1;
		}
		return d;
	}
	
	/*
	 * Returns a (1, 1, ..., 1)
	 * vector
	 */
	public static double[] ones(int length)
	{
		double [] d = new double[length-1];
		for(int j =0; j< d.length; j++)
		{
			d[j] = 0;
		}
		return d;
	}
	
	/*
	 * vector is cero
	 * vector
	 */
	
	public static boolean isVectorCero(double []x)
	{
		boolean returnable = true;
		for(double d: x)
		{
			returnable = d == 0;
			if(!returnable)
				return false;
		}
		return returnable;
	}
	
	/*
	 * Returns a (x1+y1, x2+y2, ..., xn+yn)
	 * vector
	 */
	
	public static double[] vectorialSum(double []x, double []y)
	{
		double [] d = new double[y.length];
		for(int j =0; j< x.length; j++)
		{
			d[j] = x[j] + y[j];
		}
		return d;
	}
	

	/*
	 * Returns a (x1-y1, x2-y2, ..., xn-yn)
	 * vector
	 */
	public static double[] vectorialRest(double []x, double []y)
	{
		double [] d = new double[y.length];
		for(int j =0; j< x.length; j++)
		{
			d[j] = x[j] - y[j];
		}
		
		return d;
	}
	
	/*
	 * Returns the n-file of matrix
	 */
	public static double[] getFileAt(double[][] matrix, int index)
	{
		double [] d = new double[matrix.length];
		for(int k = 0; k< d.length; k++)
		{
			d[k] = matrix[k][index];
		}
		
		
		return d;             
	}
	/*
	 * Returns a (x1-y1, x2-y2, ..., xn-yn)
	 * vector
	 */
	
	public static double productScalar(double []x, double [] y)
	{
		double d = 0;
		for(int j =0; j< x.length; j++)
		{
			d += x[j] * y[j];
		}
		return d;
	}
	
	/*
	 * Cuts a Vector to lenght dimensions
	 * */
	public static double[] subVector(double []x, int lenght)
	{
		double [] d = new double[lenght];
		for(int j =0; j< lenght; j++)
		{
			d[j] = x[j];
		}
		//SimplexUtil.printVector("voltando de multiplecar", d);
		return d;
	}
	
	/*
	 * multyplies a file by a number
	 */
	public static double[][] multipliesFileBy(double [][]x, double num, int file)
	{
		double [][] d = new double[x.length][x[0].length];
		
		for(int k=0; k< x.length; k++ )
		{
			for(int j=0; j<x[0].length; j++ )
			{
				System.out.println(x[k][j]);
				d[k][j] = j == file ? x[k][j] * num : x[k][j];
			}
		}
		
		return d;
	}
	
	
	/*
	 * Returns a (x1+scalar, x2+scalar, ..., xn+scalar)
	 * vector
	 */
	
	public static double[] scalarSum(double []x, double scalar)
	{
		double [] d = new double[x.length];
		for(int j =0; j< x.length; j++)
		{
			d[j] = x[j] + scalar;
		}
		//SimplexUtil.printVector("voltando de multiplecar", d);
		return d;
	}
	
	/*
	 * Returns a (x1*scalar, x2*scalar, ..., xn*scalar)
	 * vector
	 */
	
	public static double[] scalarMultiply(double []x, double scalar)
	{
		double [] d = new double[x.length];
		for(int j =0; j< x.length; j++)
		{
			d[j] = x[j] * scalar;
		}
		return d;
	}
	
	
	public static double normaLInfinite(double[] x)
	{
		double d = 0; 
		
		for(int j= 0; j< x.length; j++)
		{
			d = Math.max(d, Math.abs(x[j]));
		}
		System.out.println("Norma infinita " + d);
		return d;
	}
	
	public static double normaLp(double[] x, double p)
	{
		double d = 0; 
		
		for(int j= 0; j< x.length; j++)
		{
			d += Math.pow(Math.abs(x[j]), p);
		}
		return Math.pow(d, 1/p);
	}
	
	/*
	 * Returns a (x1*y1, x2*y2, ..., xn*yn)
	 * vector
	 */
	
	public static double[] vectorialMultiply(double []x, double []y)
	{
		double [] d = new double[y.length];
		for(int j =0; j< x.length; j++)
		{
			d[j] = x[j] * y[j];
		}
		return d;
	}
}
