/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator.complexEvaluator;

import de.jtem.mathExpr.evaluator.Evaluator;
//for doc-links:
import de.jtem.mathExpr.parser.UnaryOperation;
import de.jtem.mfc.field.Complex;

/**
 * The {@link Evaluator} evaluating a {@link UnaryOperation} of type '+',
 * which operand is a {@link Complex} when evaluated.
 *
 * @see ComplexType#getEvaluator(UnaryOperation,Evaluator)
 * @see UnaryOperation#getType()
 */
public class PositiveSignComplexEvaluator extends AbstractComplexEvaluator
{
	/**
	 * The <code>Evaluator</code> for the operand of the {@link
	 * UnaryOperation}.
	 */
	protected Evaluator operandEval;
	/**
	 * Creates a new <code>PositiveSignComplexEvaluator</code> with the specified
	 * operand <code>Evaluator</code>. That operand <code>Evaluator</code>
	 * provides the complex value the unary operation of type '+' will be executed
	 * on.
	 * 
	 * @param e the <code>Evaluator</code> its complex evaluation result
	 * is to be assigned.
	 */
  public PositiveSignComplexEvaluator(Evaluator e)
  {
		operandEval=e;
  }
	/**
	 * Returns the complex evaluation result of the given operand
	 * <code>Evaluator</code> as a {@link Complex}
	 * <code>Object</code>.
	 * 
	 * @return the operand as a {@link Complex} <code>Object</code>.
	 * @see #getReturnType()
	 */
  public Object evaluate()
  {
    complexValue.assign((Complex)operandEval.evaluate());
    return complexValue;
  }
}
