/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator.complexVectorEvaluator;

import de.jtem.blas.ComplexVector;
import de.jtem.mathExpr.evaluator.Evaluator;
import de.jtem.mathExpr.evaluator.Type;
//for doc-links:
import de.jtem.mathExpr.parser.Expression;

/**
 * An abstraction of all <code>Evaluators</code> evaluating an
 * {@link Expression} to a {@link ComplexVector} result.
 * 
 * @author marcel
 * @see #getReturnType()
 */
public abstract class AbstractComplexVectorEvaluator implements Evaluator
{
	/**
	 * After evaluation, the evaluated <code>ComplexVector</code> value will be
	 * stored in this field.
	 */
	ComplexVector complexVector=new ComplexVector();
	/** 
	 * Assigns <code>v</code> the evaluated <code>ComplexVector</code> value.
	 * 
	 * @param v the <code>ComplexVector</code> value the evaluated value is to be
	 * assigned to.
	 */
	public void getComplexVectorValue(ComplexVector v)
	{
		v.assign(complexVector);
	}
	/**
	 * Returns {@link ComplexVectorType#TYPE}.
	 * 
	 * @return {@link ComplexVectorType#TYPE}.
	 */
	public Type getReturnType()
	{
		return ComplexVectorType.TYPE;
	}
}
