/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2003-2009, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.mathExpr.evaluator.realEvaluator;

import de.jtem.mathExpr.evaluator.Type;
import de.jtem.mathExpr.evaluator.Variable;
//for doc-links:
import de.jtem.mathExpr.evaluator.complexEvaluator.ComplexVariable;

/**
 * A <code>RealVariable</code> is a <code>Variable</code> of {@link Type}
 * <code>RealType</code>. Although the <code>Type</code> is clear
 * you have to give it by the constructor. So you can also specify your 
 * own <code>RealType</code> implementation.<br>
 * <br>
 * <code>RealVariable</code> is used as a <code>RealEvaluator</code>
 * in an evaluator-tree. The method {@link #getRealValue(Real)} assigns
 * the constant's value to a <code>Real</code> object.
 * 
 * @author marcel
 * @see ComplexVariable
 * @see RealConstant
 */
public class RealVariable extends Variable implements RealEvaluator
{
  private Real value;
  /**
   * Creates a new <code>RealVariable</code>. <code>Type type</code> can be
   * any generalization of {@link RealType} or <code>RealType</code>
   * itself.
   * 
   * @param name the name for the new <code>Constant</code>.
   * @param value the value for the new <code>Constant</code>.
   * @param type any generalization of <code>RealType</code> or
   * <code>RealType.TYPE</code> itself.
   */
  public RealVariable(String name, Real value, Type type)
  {
    super(name, value, type);
    this.value=value;
  }
  /**
   * @see RealEvaluator#getRealValue(Real)
   */
  public void getRealValue(Real r)
  {
    r.assign(value);
  }
}
